////////////////////////////////////////////////////////////////////////
// sprite01.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Nick.h"

#define MAX_X 79
#define MAX_Y 199

void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
  __asm
    ld a, 4 (ix)
    ld b, 5 (ix)
    ld c, b
    call #0xBC32 ;SCR SET INK
    __endasm;
}

void SetPalette(const unsigned char *pPalette)
{
  unsigned char nColor = 0;

  for(nColor = 0; nColor < NUM_COLORS; nColor++)
    SetColor(nColor, pPalette[nColor]);
}

void PutSpriteMode0(unsigned char *pAddress, unsigned char nWidth, unsigned char nHeight, unsigned char *pSprite)
{
	__asm
		LD L, 4(IX) 
		LD H, 5(IX) 
		LD C, 6(IX) 
		LD B, 7(IX)            
		LD E, 8(IX) 
		LD D, 9(IX) 

		_loop_alto:
		   PUSH BC
		   LD B,C
		   PUSH HL
		_loop_ancho:
		   LD A,(DE)
		   LD (HL),A
		   INC DE
		   INC HL
		   DJNZ _loop_ancho
		   POP HL
		   LD A,H
		   ADD #0x08
		   LD H,A
		   SUB #0xC0
		   JP NC, _sig_linea
		   LD BC, #0xC050
		   ADD HL,BC
		_sig_linea:
		   POP BC
		   DJNZ _loop_alto
	__endasm;
}


////////////////////////////////////////////////////////////////////////
unsigned char char1,char2,char3,char4;

unsigned int GetTime()
{
	unsigned int nTime = 0;

	__asm
		CALL #0xBD0D ;KL TIME PLEASE
		PUSH HL
		POP DE
		LD HL, #_char3
		LD (HL), D
		LD HL, #_char4
		LD (HL), E
	__endasm;

	nTime = (char3 << 8) + char4;

	return nTime;
}
////////////////////////////////////////////////////////////////////////

enum _eDirection
{
	Direction_Left,
	Direction_Right,
	Direction_Max
};

char *aKeyFrames[Direction_Max][4];

struct _tSprite
{
	int nY;
	int nX;
	int nKey;
	unsigned int nSpeed;
	unsigned int nLastMoveTime;
	enum _eDirection eDirection;
}_tSprite;

#define NUM_SPRITES 1

struct _tSprite aSprites[NUM_SPRITES];

void main()
{
	//SCR_SET_MODE 0
	__asm
		ld a, #0
		call #0xBC0E
	__endasm;

	//SCR SET BORDER 0
  __asm
    ld b, #0 ;black
    ld c, b
    call #0xBC38
  __endasm;
  
  SetPalette(NickPalette);

	aKeyFrames[Direction_Left][0] = (char *)NickSprite;
	aKeyFrames[Direction_Left][1] = (char *)NickSprite + NICK_WIDTH * NICK_HEIGHT;
	aKeyFrames[Direction_Left][2] = (char *)NickSprite + (NICK_WIDTH * NICK_HEIGHT) * 2;
	aKeyFrames[Direction_Left][3] = (char *)NickSprite + NICK_WIDTH * NICK_HEIGHT;
	aKeyFrames[Direction_Right][0] = (char *)NickSprite + (NICK_WIDTH * NICK_HEIGHT) * 3;
	aKeyFrames[Direction_Right][1] = (char *)NickSprite + (NICK_WIDTH * NICK_HEIGHT) * 4;
	aKeyFrames[Direction_Right][2] = (char *)NickSprite + (NICK_WIDTH * NICK_HEIGHT) * 5;
	aKeyFrames[Direction_Right][3] = (char *)NickSprite + (NICK_WIDTH * NICK_HEIGHT) * 4;

	aSprites[0].eDirection = Direction_Right;
	aSprites[0].nKey = 0;
	aSprites[0].nX = 0;
	aSprites[0].nY = 75;
	aSprites[0].nSpeed = 20;
	aSprites[0].nLastMoveTime = GetTime();

	while(1)
	{
		unsigned char nSprite = 0;

		for(nSprite = 0; nSprite < NUM_SPRITES; nSprite++)
		{
			if(GetTime() - aSprites[nSprite].nLastMoveTime < aSprites[nSprite].nSpeed)
				continue;

			aSprites[nSprite].nLastMoveTime += aSprites[nSprite].nSpeed;

			//move
			aSprites[nSprite].nX += (aSprites[nSprite].eDirection == Direction_Right ? 1 : -1);

			if(aSprites[nSprite].nX <= 0)
			{
				aSprites[nSprite].nX = 0;
				aSprites[nSprite].eDirection = Direction_Right;
				aSprites[nSprite].nKey = 0;
			}

			if(aSprites[nSprite].nX >= (MAX_X - NICK_WIDTH))
			{
				aSprites[nSprite].nX = MAX_X - NICK_WIDTH;
				aSprites[nSprite].eDirection = Direction_Left;
				aSprites[nSprite].nKey = 0;
			}

			aSprites[nSprite].nKey++;

			if(aSprites[nSprite].nKey > 3)
				aSprites[nSprite].nKey = 0;

			PutSpriteMode0((unsigned char *)(0xC000 + ((aSprites[nSprite].nY / 8u) * 80u) + ((aSprites[nSprite].nY % 8u) * 2048u) + aSprites[nSprite].nX),
				NICK_WIDTH, NICK_HEIGHT, (unsigned char *)aKeyFrames[aSprites[nSprite].eDirection][aSprites[nSprite].nKey]);
		}
	}
}
////////////////////////////////////////////////////////////////////////